;
; $Id: nuv_addwcs.pro,v 1.1 2005/02/14 15:05:00 sbeland Exp $
;
;+
; NAME:
;	NUV_ADDWCS
;
; PURPOSE:
;	This procedure allows the user to select new files to be added 
;	to a list to then process then with nuv_resolution to determine
;  the wavelength coverage and adds WCS FITS keywords to the file.
;
; ONLY NUV x1d FITS file can be processed.
;
; CALLING SEQUENCE:
;	NUV_ADDWCS [, PARENT=parent]
;
; OPTIONAL INPUTS:
; 	PARENT:  Widget ID of the group leader for this widget
;
; MODIFICATION HISTORY:
; 	Written by:	SBeland CU-Boulder/CASA, February 2005
;-
;----------------------------------------------------------------------------
PRO nuv_addwcs_event, sEvent

    WIDGET_CONTROL, sEvent.id, GET_UVALUE=uval

    ; Handle KILL requests.
    IF TAG_NAMES(sEvent, /STRUCTURE_NAME) EQ 'WIDGET_KILL_REQUEST' THEN BEGIN
       ; Destroy the objects.
       WIDGET_CONTROL, sEvent.top, /DESTROY
       RETURN
    ENDIF ELSE IF TAG_NAMES(sEvent, /STRUCTURE_NAME) EQ 'WIDGET_TEXT_CH' THEN BEGIN
       RETURN
    ENDIF

    ; Get the state structure stored in the user value of the window
    WIDGET_CONTROL, GET_UVALUE=state, sEvent.top, /NO_COPY

    ; Handle other events.
    CASE uval OF
        'SELECT' : BEGIN
            ; bring up the file browser and add to list
            fileName = DIALOG_PICKFILE(GROUP=state.wBase, /MULTIPLE_FILES, /MUST_EXIST, $
                       PATH=state.initPath, GET_PATH=newpath, filter=state.initFilter)
            IF newpath NE state.initPath AND newpath NE '' THEN state.initPath = newpath

            ; If the user didn't cancel the standard file dialog
            IF STRLEN(fileName[0]) GT 0 THEN BEGIN
               ; first make sure that it is not already in the list
               IF (*state.listarr)[0] EQ "" THEN BEGIN
						PTR_FREE, state.listarr
                  state.nlist = N_ELEMENTS(fileName)
						state.listarr = PTR_NEW(fileName, /NO_COPY)
               ENDIF ELSE BEGIN
                  FOR i=0, N_ELEMENTS(fileName)-1 DO BEGIN
                     pos = WHERE(fileName[i] EQ (*state.listarr)[0:state.nlist-1])
                     IF pos[0] EQ -1 THEN BEGIN
                        *state.listarr = [*state.listarr,fileName[i]]
                        state.nlist = state.nlist + 1
                     ENDIF
                  ENDFOR
               ENDELSE
               WIDGET_CONTROL, state.wList, SET_VALUE=(*state.listarr)
               WIDGET_CONTROL, state.wProcessButton, SENSITIVE=1
               WIDGET_CONTROL, state.wClearButton, SENSITIVE=1
               WIDGET_CONTROL, state.wListLabel, $
                  SET_VALUE='Files to Process:  '+STRTRIM(STRING(state.nlist),2) 
            ENDIF
        END

        'REMOVE' : BEGIN
            ; remove selected files from the list
            pos = WIDGET_INFO(state.wList, /LIST_SELECT)
            IF pos[0] NE -1 THEN BEGIN
               (*state.listarr)[pos] = ""
               state.nlist = state.nlist - N_ELEMENTS(pos)
               IF state.nlist EQ 0 THEN BEGIN
                  pos=0
                  PTR_FREE, state.listarr
                  state.listarr = PTR_NEW("")
                  WIDGET_CONTROL, state.wRemoveButton, SENSITIVE=0
                  WIDGET_CONTROL, state.wClearButton, SENSITIVE=0
                  WIDGET_CONTROL, state.wProcessButton, SENSITIVE=0
               ENDIF ELSE BEGIN
                  newlist=(*state.listarr)[WHERE(*state.listarr ne "")]
                  PTR_FREE, state.listarr
                  state.listarr = PTR_NEW(newlist, /NO_COPY)
               ENDELSE
               WIDGET_CONTROL, state.wList, SET_VALUE=(*state.listarr)
               WIDGET_CONTROL, state.wRemoveButton, SENSITIVE=0
               WIDGET_CONTROL, state.wListLabel, $
                  SET_VALUE='Files to Process:  '+STRTRIM(STRING(state.nlist),2) 
            ENDIF

          END

        'CLEAR' : BEGIN
            ; clear all the files in the list
            PTR_FREE, state.listarr
            state.listarr = PTR_NEW('')
            state.nlist = 0
            WIDGET_CONTROL, state.wList, SET_VALUE=['']
            WIDGET_CONTROL, state.wRemoveButton, SENSITIVE=0
            WIDGET_CONTROL, state.wClearButton, SENSITIVE=0
            WIDGET_CONTROL, state.wProcessButton, SENSITIVE=0
            WIDGET_CONTROL, state.wListLabel, $
               SET_VALUE='Files to Process:  '+STRTRIM(STRING(state.nlist),2) 
          END

        'PROCESS' : BEGIN
             ; disable the buttons while processing
             WIDGET_CONTROL, state.wRemoveButton, SENSITIVE=0
             WIDGET_CONTROL, state.wClearButton, SENSITIVE=0
             WIDGET_CONTROL, state.wProcessButton, SENSITIVE=0
             WIDGET_CONTROL, state.wCancelButton, SENSITIVE=0
             WIDGET_CONTROL, state.wSelectButton, SENSITIVE=0

             ; initialize a progress meter
             p_meter = progress_meter(group_leader=sEvent.top,TEXT='Extracting the Items')

             ; loop through every file 
             progress=0
             nfiles=state.nlist
             out_data=strarr(n_elements(fits_keywords)+2,nfiles)
             max_rms = 10.0   ; maximum allowable rms deviation in pixels for xcorrel

             WHILE state.nlist NE 0 DO BEGIN
                ; check if cancel button was pressed
                IF (*p_meter).cancelFlag THEN BEGIN
                   progress_Destroy, p_meter
                   Obj_Destroy, progressTimer
                   WIDGET_CONTROL, state.wCancelButton, SENSITIVE=1
                   WIDGET_CONTROL, state.wSelectButton, SENSITIVE=1
                   WIDGET_CONTROL, state.wRemoveButton, SENSITIVE=1
                   WIDGET_CONTROL, state.wClearButton, SENSITIVE=1
                   IF WIDGET_INFO(sEvent.top, /VALID_ID) THEN $
                      WIDGET_CONTROL, SET_UVALUE = state, sEvent.top, /NO_COPY
                   RETURN
                ENDIF

                ; get the wavelength solution for the file
                filename = (*state.listarr)[0]
                fdecomp, filename, disk, dir, fname, ext
                d0 = mrdfits(filename,0,h0,/silent)
                d1 = mrdfits(filename,1,h1,/silent)
                ; get the maximum value in the spectrum and set minpeak to 0.1 of max
                minpeak = max(d1.y) * 0.1
                solution = nuv_resolution(filename,/noplot,/nosave, minpeak=minpeak,waveorder=1,maxresidual=10)

                if size(solution, /TNAME) eq "STRUCT" then begin
                    print, fname, solution.coeffs[0], solution.coeffs[1], solution.rms
                    if solution.rms le max_rms then begin
                        ; only update the WCS if we get a decent fit
                        ; open the file and write the wcs
                        ctype = sxpar(h1,'CTYPE1', count=count, comment=comment)
                        if count eq 0 then comment = 'the coordinate type for the first axis' 
                        sxaddpar, h1, 'CTYPE1', 'WAVE', comment

                        cunit = sxpar(h1,'CUNIT1', count=count, comment=comments)
                        if count eq 0 then comment = 'units of first coordinate value'
                        sxaddpar, h1, 'CUNIT1', 'Angstrom', comment

                        crval = sxpar(h1,'CRVAL1', count=count, comment=comments)
                        if count eq 0 then comment = 'first axis value at reference pixel'
                        sxaddpar, h1, 'CRVAL1', solution.coeffs[0], comment

                        crpix = sxpar(h1,'CRPIX1', count=count, comment=comments)
                        if count eq 0 then comment = 'x-coordinate of reference pixel'
                        sxaddpar, h1, 'CRPIX1', 0.0, comment

                        cd1_1 = sxpar(h1,'CD1_1', count=count, comment=comments)
                        if count eq 0 then comment = 'partial of first axis coordinate w.r.t. x (A/pix)'
                        sxaddpar, h1, 'CD1_1', solution.coeffs[1], comment

                        cd1_1 = sxpar(h1,'WCSRMS_1', count=count, comment=comments)
                        if count eq 0 then comment = 'RMS of fitted CRVAL1 and CD1_1'
                        sxaddpar, h1, 'WCSRMS_1', solution.rms, comment

                        mwrfits,nothing,filename,h0,/create,/silent,/no_comment
                        mwrfits,d1,filename,h1,/silent,/no_comment
                    endif else begin
                        print, fname + " ->  No good fit was found (rms too high)!! "
                    endelse
                endif else begin
                    print, fname + " ->  No fit was found !! "
                endelse

                ; set the percentage done
                progress=progress+1
                progress_update, p_meter, FLOAT(progress)*100.0 / FLOAT(nfiles)

                ; remove the entry from the list
                (*state.listarr)[0] = ""
                state.nlist = state.nlist - 1
                IF state.nlist EQ 0 THEN BEGIN
                   PTR_FREE, state.listarr
                   state.listarr = PTR_NEW('')
                   WIDGET_CONTROL, state.wRemoveButton, SENSITIVE=0
                   WIDGET_CONTROL, state.wClearButton, SENSITIVE=0
                   WIDGET_CONTROL, state.wCancelButton, SENSITIVE=1
                   WIDGET_CONTROL, state.wSelectButton, SENSITIVE=1
                   WIDGET_CONTROL, state.wProcessButton, SENSITIVE=0
                   WIDGET_CONTROL, state.wList, SET_VALUE=['']
                ENDIF ELSE BEGIN
                   newlist = (*state.listarr)[WHERE(*state.listarr NE "")]
                   PTR_FREE, state.listarr
                   state.listarr = PTR_NEW(newlist, /NO_COPY)
                   WIDGET_CONTROL, state.wList, SET_VALUE=*state.listarr
                ENDELSE

             ENDWHILE

             ; get rid of meter
             progress_Destroy, p_meter

             ; re-enable the buttons
             IF WIDGET_INFO(state.wList, /LIST_SELECT) EQ -1 THEN BEGIN
                WIDGET_CONTROL, state.wRemoveButton, SENSITIVE=0 
                WIDGET_CONTROL, state.wClearButton, SENSITIVE=0
             ENDIF ELSE BEGIN
                WIDGET_CONTROL, state.wRemoveButton, SENSITIVE=1
                WIDGET_CONTROL, state.wClearButton, SENSITIVE=1
             ENDELSE
             WIDGET_CONTROL, state.wListLabel, $
                SET_VALUE='Files to Process:  '+STRTRIM(STRING(state.nlist),2) 
           
             ; we can now exit this tool
             WIDGET_CONTROL, sEvent.top, /DESTROY
          END

        'CANCEL' : BEGIN
            ; quit without converting
            WIDGET_CONTROL, sEvent.top, /DESTROY
          END

        'LIST' : BEGIN
            ; a file was selected from the list
            pos = WIDGET_INFO(state.wList, /LIST_SELECT)
            IF pos[0] NE -1 THEN BEGIN
               WIDGET_CONTROL, state.wRemoveButton, SENSITIVE=1
            ENDIF
        END

        ELSE: 
            ; ignore all other events
    ENDCASE

    ; Reset the windows user value to the updated state structure
    IF WIDGET_INFO(sEvent.top, /VALID_ID) THEN $
       WIDGET_CONTROL, SET_UVALUE = state, sEvent.top, /NO_COPY

END

;----------------------------------------------------------------------------
PRO nuv_addwcs, PARENT=parent

    ; Create the widgets.
    xsize = 130

    title = 'NUV_ADDWCS'
    IF KEYWORD_SET(PARENT) THEN $
       wBase = WIDGET_BASE(GROUP_LEADER=parent, /ROW, $
                           TITLE=title, /TLB_KILL_REQUEST_EVENTS, TLB_FRAME_ATTR=1) $
    ELSE $
       wBase = WIDGET_BASE(/ROW, $
                           TITLE=title, /TLB_KILL_REQUEST_EVENTS, TLB_FRAME_ATTR=1)
 
    wGuiBase      = WIDGET_BASE(wBase, COL=2)
    wListBase     = WIDGET_BASE(wGuiBase, /COLUMN)
    wListLabel    = WIDGET_LABEL(wListBase, VALUE='Files to Process:  0', $
                    /ALIGN_LEFT, /DYNAMIC_RESIZE)
    wList         = WIDGET_LIST(wListBase, /MULTIPLE, XSIZE=50, YSIZE=11, UVALUE='LIST')

    wButtons      = WIDGET_BASE(wGuiBase, /COLUMN, SPACE=53, YPAD=27)
    wTopButtons   = WIDGET_BASE(wButtons,/COLUMN, SPACE=10, frame=1)
    wSelectButton = WIDGET_BUTTON(wTopButtons, XSIZE=xsize, VALUE='Select', UVALUE='SELECT')
    wRemoveButton = WIDGET_BUTTON(wTopButtons, XSIZE=xsize, VALUE='Remove', UVALUE='REMOVE')
    wClearButton  = WIDGET_BUTTON(wTopButtons, XSIZE=xsize, VALUE='Clear', UVALUE='CLEAR')

    wBottomButtons= WIDGET_BASE(wButtons,/COLUMN, /BASE_ALIGN_BOTTOM, SPACE=10, frame=1)
    wProcessButton= WIDGET_BUTTON(wBottomButtons, XSIZE=xsize, VALUE='Process', UVALUE='PROCESS')
    wCancelButton = WIDGET_BUTTON(wBottomButtons, XSIZE=xsize, VALUE='Cancel', UVALUE='CANCEL')

    state = { $
             wBase: wBase, $
             wList: wList, $
             wListLabel: wListLabel, $
             wSelectButton: wSelectButton, $
             wRemoveButton: wRemoveButton, $
             wClearButton: wClearButton, $
             wProcessButton: wProcessButton, $
             wCancelButton: wCancelButton, $
             listarr: PTR_NEW(''), $
             nlist: 0, $
             initPath: cedar_getenv('CEDAR_REFERENCE_PATH'), $
             initFilter: '*x1d*.fits', $
             out_data:PTR_NEW() $
           }

    ; set the initial browsing path to the default raw dir set by environment
    path = CEDAR_GETENV('CEDAR_FITS_ROOT')
    IF path NE '' THEN state.initPath=path

    ; Save the state structure in the window's user value
    WIDGET_CONTROL, wBase, SET_UVALUE=state

    WIDGET_CONTROL, wRemoveButton, SENSITIVE=0
    WIDGET_CONTROL, wClearButton, SENSITIVE=0
    WIDGET_CONTROL, wProcessButton, SENSITIVE=0

    WIDGET_CONTROL, wBase, /REALIZE

    XMANAGER, 'NUV_ADDWCS', wBase, /NO_BLOCK

END
