; plot a series of 10 plots on one page with the various bandpass regions used 
; for the relative nuv grating efficiencies measurements

pro plot_nuv_eff, files=files

   if n_elements(files) eq 0 then begin
       path = CEDAR_GETENV('CEDAR_FITS_ROOT')
       files = dialog_pickfile(/multi, filter='*x1d*fits', path=path)
       if files[0] eq '' then return
   endif

   wregion = [$
        {grating:'G185M', cenwave:1986, wr:[2093.0,2105.0]}, $
        {grating:'G185M', cenwave:2010, wr:[2126.0,2130.0]}, $
        {grating:'G225M', cenwave:2186, wr:[2093.0,2105.0]}, $
        {grating:'G225M', cenwave:2217, wr:[2126.0,2130.0]}, $
        {grating:'G225M', cenwave:2390, wr:[2485.0,2500.0]}, $
        {grating:'G225M', cenwave:2410, wr:[2503.0,2519.0]}, $
        {grating:'G285M', cenwave:2617, wr:[2485.0,2500.0]}, $
        {grating:'G285M', cenwave:2637, wr:[2503.0,2519.0]}, $
        {grating:'G230L', cenwave:2635, wr:[2478.0,2533.0]}, $
        {grating:'G230L', cenwave:3360, wr:[2120.0,2140.0]} $
        ]


   nfiles = n_elements(files)
   if nfiles ne 10 then begin
       print,'We are expecting 10 files!  Only '+strtrim(string(nfiles),2) + ' selected (exiting).'
       return
   endif

   window, xs=2000, ys=1500, /pixmap, /free   ;create pixmap
   pixmapid = !d.window
   wset, pixmapid
   tek_color
   !p.background=1
   !p.charsize=3.0

   xtitle='Wavelength (A)'
   ytitle=''
   !p.multi=[0,2,5]
   for ff=0, nfiles-1 do begin
       ; read the data
       data=mrdfits(files[ff],0,h0,/silent)
       data=mrdfits(files[ff],1,h1,/silent)
       header=[h0,h1]
       grating=sxpar(header,'OPT_ELEM')
       cenwave = sxpar(header,'CENWAVE')
       crval = sxpar(header,'CRVAL1')
       crpix = sxpar(header,'CRPIX1')
       cd1_1 = sxpar(header,'CD1_1')
       wave = crval + cd1_1 * (data.x - crpix)
       p = where(wregion[*].cenwave eq cenwave,count)
       x = [wregion[p].wr[0], wregion[p].wr[0], wregion[p].wr[1], wregion[p].wr[1]]
       label = '['+strtrim(string(x[0],format='(F6.1)'),2)+'->'+strtrim(string(x[2],format='(F6.1)'),2)+']'
       title = strupcase(strtrim(grating,2)) + ',  CENWAVE=' + strtrim(string(cenwave),2)
       title = title + '   ' + label
       plot,wave,data.y, color=0, xtitle=xtitle,ytitle=ytitle,title=title, /xstyle, /ystyle

       ; plot the dashed lines for the bandpass region
       if count eq 0 then continue
       y = [!y.crange[0],!y.crange[1],!y.crange[0],!y.crange[1]]
       oplot,x[0:1],y[0:1], color=0, linestyle=2, thick=2.0
       oplot,x[2:3],y[2:3], color=0, linestyle=2, thick=2.0
   endfor
   filename='plot_nuv_eff.jpg'
   img=tvrd(true=3)
   write_jpeg,filename,img,true=3
   
   ; device,/close
   set_plot,'X'

   !p.multi=0

end
